@@ -26,7 +26,8 @@ import logging
 
 from liquidctl.driver.usb import UsbHidDriver
 from liquidctl.error import NotSupportedByDevice
-from liquidctl.util import clamp, normalize_profile, interpolate_profile
+from liquidctl.util import clamp, normalize_profile, interpolate_profile, \
+                           map_direction
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -104,9 +105,9 @@ class Kraken2(UsbHidDriver):
         self._connected = False
 
     def connect(self, **kwargs):
-        super().connect(**kwargs)
+        ret = super().connect(**kwargs)
         self._connected = True
-        return self
+        return ret
 
     def disconnect(self, **kwargs):
         super().disconnect(**kwargs)
@@ -147,23 +148,16 @@ class Kraken2(UsbHidDriver):
         if not self.supports_lighting:
             raise NotSupportedByDevice()
 
-        channel = channel.lower()
-        mode = mode.lower()
-        speed = speed.lower()
-        direction = direction.lower()
-
         if mode == 'super':
             _LOGGER.warning('deprecated mode, move to super-fixed, super-breathing or super-wave')
             mode = 'super-fixed'
         if 'backwards' in mode:
-            _LOGGER.warning('deprecated mode, move to direction=backwards option')
+            _LOGGER.warning('deprecated mode, move to direction=backward option')
             mode = mode.replace('backwards-', '')
             direction = 'backward'
 
         mval, mod2, mod4, mincolors, maxcolors, ringonly = _COLOR_MODES[mode]
-
-        if direction == 'backward':
-            mod2 += 0x10
+        mod2 += map_direction(direction, 0, 0x10)
 
         if ringonly and channel != 'ring':
             _LOGGER.warning('mode=%s unsupported with channel=%s, dropping to ring',
