@@ -3,7 +3,7 @@
 Supported devices:
 
 - NZXT Kraken X (X53, X63 and Z73)
-- NZXT Kraken Z (Z63 and Z73); no OLED screen control yet
+- NZXT Kraken Z (Z53, Z63 and Z73); no LCD screen control yet
 
 Copyright (C) 2020–2021  Tom Frey, Jonas Malaco and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
@@ -349,7 +349,7 @@ class KrakenZ3(KrakenX3):
     """Fourth-generation Kraken Z liquid cooler."""
 
     SUPPORTED_DEVICES = [
-        (0x1e71, 0x3008, None, 'NZXT Kraken Z (Z63 or Z73) (experimental)', {
+        (0x1e71, 0x3008, None, 'NZXT Kraken Z (Z53, Z63 or Z73) (experimental)', {
             'speed_channels': _SPEED_CHANNELS_KRAKENZ,
             'color_channels': _COLOR_CHANNELS_KRAKENZ,
         })
