@@ -51,7 +51,7 @@ The subclass constructor can generally be kept unaware of the implementation
 details of the device parameter, and find_supported_devices already accepts
 keyword arguments and forwards them to the driver constructor.
 
-Copyright (C) 2019–2020  Jonas Malaco and contributors
+Copyright (C) 2019–2021  Jonas Malaco and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
@@ -60,6 +60,9 @@ import sys
 
 import usb
 try:
+    # The hidapi package, depending on how it's compiled, exposes one or two
+    # top level modules: hid and, optionally, hidraw.  When both are available,
+    # hid will be a libusb-based fallback implementation, and we prefer hidraw.
     import hidraw as hid
 except ModuleNotFoundError:
     import hid
@@ -67,7 +70,7 @@ except ModuleNotFoundError:
 from liquidctl.driver.base import BaseDriver, BaseBus, find_all_subclasses
 from liquidctl.util import LazyHexRepr
 
-LOGGER = logging.getLogger(__name__)
+_LOGGER = logging.getLogger(__name__)
 
 
 class BaseUsbDriver(BaseDriver):
@@ -103,7 +106,7 @@ class BaseUsbDriver(BaseDriver):
             consargs = devargs.copy()
             consargs.update(kwargs)
             dev = cls(handle, description, **consargs)
-            LOGGER.debug('instanced driver for %s', description)
+            _LOGGER.debug('instanced driver for %s', description)
             yield dev
 
     def __init__(self, device, description, **kwargs):
@@ -113,6 +116,7 @@ class BaseUsbDriver(BaseDriver):
     def connect(self, **kwargs):
         """Connect to the device."""
         self.device.open()
+        return self
 
     def disconnect(self, **kwargs):
         """Disconnect from the device."""
@@ -184,8 +188,8 @@ class UsbHidDriver(BaseUsbDriver):
         # instantiated with a usb.core.Device
         if isinstance(device, usb.core.Device):
             clname = self.__class__.__name__
-            LOGGER.warning('constructing a %s instance from a usb.core.Device has been deprecated, '
-                           'use %s.find_supported_devices() or pass a HidapiDevice handle', clname, clname)
+            _LOGGER.warning('constructing a %s instance from a usb.core.Device has been deprecated, '
+                            'use %s.find_supported_devices() or pass a HidapiDevice handle', clname, clname)
             usbdev = device
             hidinfo = next(info for info in hid.enumerate(usbdev.idVendor, usbdev.idProduct)
                            if info['serial_number'] == usbdev.serial_number)
@@ -237,30 +241,47 @@ class PyUsbDevice:
         Ensure the device is configured and replace the kernel kernel on the
         selected interface, if necessary.
         """
+
+        # we assume the device is already configured, there is only one
+        # configuration, or the first one is desired
+
         try:
             cfg = self.usbdev.get_active_configuration()
-        except usb.core.USBError:
-            LOGGER.debug('setting the (first) configuration')
-            self.usbdev.set_configuration()  # assume the first configuration
-            # FIXME device or handle might not be ready for use after set_configuration()
-            cfg = self.usbdev.get_active_configuration()
+        except usb.core.USBError as err:
+            if err.args[0] == 'Configuration not set':
+                _LOGGER.debug('setting the (first) configuration')
+                self.usbdev.set_configuration()
+                # FIXME device or handle might not be ready for use yet
+                cfg = self.usbdev.get_active_configuration()
+            else:
+                raise
+
         self.bInterfaceNumber = self._select_interface(cfg)
-        LOGGER.debug('selected interface: %d', self.bInterfaceNumber)
+        _LOGGER.debug('selected interface: %d', self.bInterfaceNumber)
+
         if (sys.platform.startswith('linux') and
                 self.usbdev.is_kernel_driver_active(self.bInterfaceNumber)):
-            LOGGER.debug('replacing stock kernel driver with libusb')
+            _LOGGER.debug('replacing stock kernel driver with libusb')
             self.usbdev.detach_kernel_driver(self.bInterfaceNumber)
             self._attached = True
 
     def claim(self):
         """Explicitly claim the device from other programs."""
-        LOGGER.debug('explicitly claim interface')
+        _LOGGER.debug('explicitly claim interface')
         usb.util.claim_interface(self.usbdev, self.bInterfaceNumber)
 
     def release(self):
         """Release the device to other programs."""
-        LOGGER.debug('ensure interface is released')
-        usb.util.release_interface(self.usbdev, self.bInterfaceNumber)
+        if sys.platform == 'win32':
+            # on Windows we need to release the entire device for other
+            # programs to be able to access it
+            _LOGGER.debug('explicitly release device')
+            usb.util.dispose_resources(self.usbdev)
+        else:
+            # on Linux, and possibly on Mac and BSDs, releasing the specific
+            # interface is enough
+            _LOGGER.debug('explicitly release interface')
+            usb.util.release_interface(self.usbdev, self.bInterfaceNumber)
 
     def close(self):
         """Disconnect from the device.
@@ -269,24 +290,24 @@ class PyUsbDevice:
         """
         self.release()
         if self._attached:
-            LOGGER.debug('restoring stock kernel driver')
+            _LOGGER.debug('restoring stock kernel driver')
             self.usbdev.attach_kernel_driver(self.bInterfaceNumber)
             self._attached = False
 
     def read(self, endpoint, length, timeout=None):
         """Read from endpoint."""
         data = self.usbdev.read(endpoint, length, timeout=timeout)
-        LOGGER.debug('read %d bytes: %r', len(data), LazyHexRepr(data))
+        _LOGGER.debug('read %d bytes: %r', len(data), LazyHexRepr(data))
         return data
 
     def write(self, endpoint, data, timeout=None):
         """Write to endpoint."""
-        LOGGER.debug('writting %d bytes: %r', len(data), LazyHexRepr(data))
+        _LOGGER.debug('writting %d bytes: %r', len(data), LazyHexRepr(data))
         return self.usbdev.write(endpoint, data, timeout=timeout)
 
     def ctrl_transfer(self, *args, **kwargs):
         """Submit a contrl transfer."""
-        LOGGER.debug('sending control transfer with %r, %r', args, kwargs)
+        _LOGGER.debug('sending control transfer with %r, %r', args, kwargs)
         return self.usbdev.ctrl_transfer(*args, **kwargs)
 
     @classmethod
@@ -317,7 +338,7 @@ class PyUsbDevice:
 
     @property
     def bus(self):
-        return 'usb{}'.format(self.usbdev.bus)  # follow Linux model
+        return f'usb{self.usbdev.bus}'  # follow Linux model
 
     @property
     def address(self):
@@ -383,7 +404,7 @@ class HidapiDevice:
         discarded = 0
         while self.hiddev.read(max_length=1, timeout_ms=timeout_ms):
             discarded += 1
-        LOGGER.debug('discarded %d previously enqueued reports', discarded)
+        _LOGGER.debug('discarded %d previously enqueued reports', discarded)
 
     def read(self, length):
         """Read raw report from HID.
@@ -397,7 +418,7 @@ class HidapiDevice:
         """
         self.hiddev.set_nonblocking(False)
         data = self.hiddev.read(length)
-        LOGGER.debug('read %d bytes: %r', len(data), LazyHexRepr(data))
+        _LOGGER.debug('read %d bytes: %r', len(data), LazyHexRepr(data))
         return data
 
     def write(self, data):
@@ -410,13 +431,13 @@ class HidapiDevice:
         > first byte should be set to 0. The report data itself should begin
         > at the second byte.
         """
-        LOGGER.debug('writting report 0x%02x with %d bytes: %r', data[0],
-                     len(data) - 1, LazyHexRepr(data, start=1))
+        _LOGGER.debug('writting report 0x%02x with %d bytes: %r', data[0],
+                      len(data) - 1, LazyHexRepr(data, start=1))
         res = self.hiddev.write(data)
         if res < 0:
-                raise OSError('Could not write to device')
+            raise OSError('Could not write to device')
         if res != len(data):
-            LOGGER.debug('wrote %d total bytes, expected %d', res, len(data))
+            _LOGGER.debug('wrote %d total bytes, expected %d', res, len(data))
         return res
 
     def get_feature_report(self, report_id, length):
@@ -430,8 +451,8 @@ class HidapiDevice:
         byte.
         """
         data = self.hiddev.get_feature_report(report_id, length)
-        LOGGER.debug('got feature report 0x%02x with %d bytes: %r', data[0],
-                     len(data) - 1, LazyHexRepr(data, start=1))
+        _LOGGER.debug('got feature report 0x%02x with %d bytes: %r', data[0],
+                      len(data) - 1, LazyHexRepr(data, start=1))
         return data
 
     def send_feature_report(self, data):
@@ -444,13 +465,13 @@ class HidapiDevice:
         > first byte should be set to 0. The report data itself should begin
         > at the second byte.
         """
-        LOGGER.debug('sending feature report 0x%02x with %d bytes: %r',
-                     data[0], len(data) - 1, LazyHexRepr(data, start=1))
+        _LOGGER.debug('sending feature report 0x%02x with %d bytes: %r',
+                      data[0], len(data) - 1, LazyHexRepr(data, start=1))
         res = self.hiddev.send_feature_report(data)
         if res < 0:
-                raise OSError('Could not send feature report to device')
+            raise OSError('Could not send feature report to device')
         if res != len(data):
-            LOGGER.debug('sent %d total bytes, expected %d', res, len(data))
+            _LOGGER.debug('sent %d total bytes, expected %d', res, len(data))
         return res
 
     @classmethod
@@ -494,19 +515,23 @@ class HidapiDevice:
 
 
 class HidapiBus(BaseBus):
-    def find_devices(self, vendor=None, product=None, bus=None, address=None, **kwargs):
-        """Find compatible USB HID devices."""
+    def find_devices(self, vendor=None, product=None, bus=None, address=None,
+                     usb_port=None, **kwargs):
+        """Find compatible HID devices."""
         handles = HidapiDevice.enumerate(hid, vendor, product)
-        drivers = sorted(find_all_subclasses(UsbHidDriver), key=lambda x: x.__name__)
-        LOGGER.debug('searching %s (api=%s, drivers=[%s])', self.__class__.__name__, hid.__name__,
-                     ', '.join(map(lambda x: x.__name__, drivers)))
+        drivers = sorted(find_all_subclasses(UsbHidDriver),
+                         key=lambda x: (x.__module__, x.__name__))
+        _LOGGER.debug('searching %s (%s)', self.__class__.__name__,
+                      ', '.join(map(lambda x: x.__name__, drivers)))
         for handle in handles:
             if bus and handle.bus != bus:
                 continue
             if address and handle.address != address:
                 continue
-            LOGGER.debug('probing drivers for device %04x:%04x', handle.vendor_id,
-                         handle.product_id)
+            if usb_port and handle.port != usb_port:
+                continue
+            _LOGGER.debug('found HID device %04x:%04x', handle.vendor_id,
+                          handle.product_id)
             for drv in drivers:
                 yield from drv.probe(handle, vendor=vendor, product=product, **kwargs)
 
@@ -515,9 +540,10 @@ class PyUsbBus(BaseBus):
     def find_devices(self, vendor=None, product=None, bus=None, address=None,
                      usb_port=None, **kwargs):
         """ Find compatible regular USB devices."""
-        drivers = sorted(find_all_subclasses(UsbDriver), key=lambda x: x.__name__)
-        LOGGER.debug('searching %s (drivers=[%s])', self.__class__.__name__,
-                     ', '.join(map(lambda x: x.__name__, drivers)))
+        drivers = sorted(find_all_subclasses(UsbDriver),
+                         key=lambda x: (x.__module__, x.__name__))
+        _LOGGER.debug('searching %s (%s)', self.__class__.__name__,
+                      ', '.join(map(lambda x: x.__name__, drivers)))
         for handle in PyUsbDevice.enumerate(vendor, product):
             if bus and handle.bus != bus:
                 continue
@@ -525,7 +551,7 @@ class PyUsbBus(BaseBus):
                 continue
             if usb_port and handle.port != usb_port:
                 continue
-            LOGGER.debug('probing drivers for device %04x:%04x', handle.vendor_id,
-                         handle.product_id)
+            _LOGGER.debug('found USB device %04x:%04x', handle.vendor_id,
+                          handle.product_id)
             for drv in drivers:
                 yield from drv.probe(handle, vendor=vendor, product=product, **kwargs)
