@@ -1,11 +1,22 @@
 """Standardized liquidctl errors.
 
-Copyright (C) 2020–2020  Jonas Malaco and contributors
+Copyright (C) 2020–2021  Jonas Malaco and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
+class ExpectationNotMet(Exception):
+    """Unstable."""
+    pass
+
+
 class NotSupportedByDevice(Exception):
     pass
 
+
 class NotSupportedByDriver(Exception):
     pass
+
+
+class UnsafeFeaturesNotEnabled(Exception):
+    """Unstable."""
+    pass
